#include <Arduino.h>
// https://github.com/RobTillaart/PCA9685_RT
#include "PCA9685.h"

#include <ESP8266WiFi.h>
#include <ESP8266WiFiMulti.h>

#include <ESP8266WebServer.h>

#include <ESP8266mDNS.h>
#include <WiFiUdp.h>
#include <ArduinoOTA.h>

#include <EEPROM.h>

#include <Wire.h>
#include <TTP229.h>

TTP229 ttp229;

PCA9685 PCA(0x40);

ESP8266WiFiMulti wifiMulti;
ESP8266WebServer server(80);

const char* ssid1     = "SSID1";
const char* password1 = "PASS1";
const char* ssid2     = "SSID2";
const char* password2 = "PASS2";

const byte led = 2;

unsigned long timing; // service timestamp
int tlim = 7000; // valve state change time

int klim = 2000; // key press timing

bool state[8];
bool serv[8];
unsigned long ktime[16] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}; // key press timestamp

void setup() {
pinMode(led, OUTPUT);  

// wifi connection section
  WiFi.mode(WIFI_OFF);        //Prevents reconnection issue (taking too long to connect)
  delay(1000);
  WiFi.mode(WIFI_STA);        //This line hides the viewing of ESP as wifi hotspot
  // WiFi.begin(ssid, password);     //Connect to your WiFi router
  wifiMulti.addAP(ssid1, password1);
  wifiMulti.addAP(ssid2, password2);
 
// Wait for connection
  while (wifiMulti.run() != WL_CONNECTED) {
    digitalWrite(led, HIGH);
    delay(250);
    digitalWrite(led, LOW);
    delay(250);
  }
    pinMode(led, INPUT);  

// OTA section
  // Port defaults to 8266
  // ArduinoOTA.setPort(8266);

  // Hostname defaults to esp8266-[ChipID]
  // ArduinoOTA.setHostname("pzem_meter");
  char tmp[15];
  sprintf(tmp, "valve-%06x", ESP.getChipId());
  ArduinoOTA.setHostname(tmp);

  // No authentication by default
  ArduinoOTA.setPassword("PASSWORD");

  // Password can be set with it's md5 value as well
  // MD5(admin) = 21232f297a57a5a743894a0e4a801fc3
  // ArduinoOTA.setPasswordHash("21232f297a57a5a743894a0e4a801fc3");

  ArduinoOTA.begin();

// HTTP server setup
  server.on("/", handle_OnConnect);
  
  server.on("/serv0on", handle_serv0on);
  server.on("/serv0serv", handle_serv0serv);
  server.on("/serv0off", handle_serv0off);

  server.on("/serv1on", handle_serv1on);
  server.on("/serv1serv", handle_serv1serv);
  server.on("/serv1off", handle_serv1off);
  
  server.on("/serv2on", handle_serv2on);
  server.on("/serv2serv", handle_serv2serv);
  server.on("/serv2off", handle_serv2off);
  
  server.on("/serv3on", handle_serv3on);
  server.on("/serv3serv", handle_serv3serv);
  server.on("/serv3off", handle_serv3off);
  
  server.on("/serv4on", handle_serv4on);
  server.on("/serv4serv", handle_serv4serv);
  server.on("/serv4off", handle_serv4off);
  
  server.on("/serv5on", handle_serv5on);
  server.on("/serv5serv", handle_serv5serv);
  server.on("/serv5off", handle_serv5off);
  
  server.on("/serv6on", handle_serv6on);
  server.on("/serv6serv", handle_serv6serv);
  server.on("/serv6off", handle_serv6off);
  
  server.on("/serv7on", handle_serv7on);
  server.on("/serv7serv", handle_serv7serv);
  server.on("/serv7off", handle_serv7off);
  
  server.on("/allon", handle_allon);
  server.on("/alloff", handle_alloff);
  server.on("/allserv", handle_allserv);

 server.on("/mainon", handle_mainon);
 server.on("/mainoff", handle_mainoff);
 server.on("/mainserv", handle_mainserv);

 server.on("/polivon", handle_polivon);
 server.on("/polivoff", handle_polivoff);
 server.on("/polivserv", handle_polivserv);

  server.onNotFound(handle_NotFound);

  server.begin();

  // begin(sda, scl)
  Wire.begin(2, 0);
  PCA.begin(2, 0);

  EEPROM.begin(3);

  for (int e = 0; e < 3; ++e) {

    if(EEPROM.read(e) == 1)
    {
      state[e] = HIGH;
    }
    else
    {
      state[e] = LOW;
    }
  }

  for (int b = 3; b < 7; ++b) {
    state[b] = LOW;
  }
}

void loop() {
  while (wifiMulti.run() != WL_CONNECTED) {
    pinMode(led, OUTPUT);
    digitalWrite(led, HIGH);
    delay(250);
    digitalWrite(led, LOW);
    delay(250);
  }
  pinMode(led, INPUT);
  
  delay(100);
  server.handleClient();
  ArduinoOTA.handle();

  for (int s = 0; s < 8; ++s) {
    if(state[s]) {
      PCA.digitalWrite(s*2, HIGH);
      PCA.digitalWrite(s*2+1, LOW);
       if (s < 3) {
         if (EEPROM.read(s) != 1)
         {
          EEPROM.write(s, 1);
          EEPROM.commit();
         }
       }
     }
      else
     {
      PCA.digitalWrite(s*2, LOW);
      PCA.digitalWrite(s*2+1, HIGH);
      if (s < 3) {
       if (EEPROM.read(s) != 0)
        {
          EEPROM.write(s, 0);
          EEPROM.commit();
        }
       }
      }

      if (serv[s] == 1) {
        if (millis() - timing > tlim)
        {
          state[s] = !state[s];
          serv[s] = 0;
        }
      }
  }

  ttp229.readKeys();
  for (int k = 0; k < 16; ++k) {
    if (ttp229.isKeyDown(k)) {
      ktime[k] = millis();
    }
    
    if (ttp229.isKeyUp(k)) {
      ktime[k] = 0;
    }
    
  
    if (ktime[k] != 0) {
      if (millis() - ktime[k] > klim) {
        int d = k/2;
        int m;
          if (d == 0) {
            m = k; // key 0 and 1 for valve 0
           } else if (d == 1) {
            m = k-2; // key 1 and 2 for valve 1
           } else {
            m = k%d; // keys 3...n
          }
    
        if (m == 0) {
         state[d] = LOW;
         } else {
         state[d] = HIGH;
        }
        ktime[k]=0;
      }
    }
  } 
}
  
// print uptime
String millis2time(){
 String Time="";
 unsigned long ss;
 byte mm, hh;
 ss = millis()/1000;
 hh = ss/3600;
 mm = (ss-hh*3600)/60;
 ss = (ss-hh*3600)-mm*60;
 if (hh<10) Time+="0";
 Time += (String)hh + ":";
 if (mm<10) Time += "0";
 Time += (String)mm + ":";
 if (ss<10) Time += "0";
 Time += (String)ss;
 return Time;
}

void handle_NotFound() {
    server.send(404, "text/plain", "Not found");
}

void handle_OnConnect() {
  server.send(200, "text/html", SendHTML());
}

void handle_serv0on()
{
  state[0] = HIGH;
  server.send(200, "text/html", SendHTML());
}

void handle_serv0off()
{
  state[0] = LOW;
  server.send(200, "text/html", SendHTML());
}

void handle_serv0serv()
{
       serv[0] = 1;
       state[0] = !state[0];
       timing = millis();
       server.send(200, "text/html", SendHTML());
}

void handle_serv1on()
{
  state[1] = HIGH;
server.send(200, "text/html", SendHTML());
}

void handle_serv1off()
{
  state[1] = LOW;
server.send(200, "text/html", SendHTML());
}

void handle_serv1serv()
{
       serv[1] = 1;
       state[1] = !state[1];
       timing = millis();
       server.send(200, "text/html", SendHTML());
}

void handle_serv2on()
{
  state[2] = HIGH;
server.send(200, "text/html", SendHTML());
}

void handle_serv2off()
{
  state[2] = LOW;
server.send(200, "text/html", SendHTML());
}

void handle_serv2serv()
{
       serv[2] = 1;
       state[2] = !state[2];
       timing = millis();
       server.send(200, "text/html", SendHTML());
}

void handle_serv3on()
{
  state[3] = HIGH;
server.send(200, "text/html", SendHTML());
}

void handle_serv3off()
{
  state[3] = LOW;
server.send(200, "text/html", SendHTML());
}

void handle_serv3serv()
{
       serv[3] = 1;
       state[3] = !state[3];
       timing = millis();
       server.send(200, "text/html", SendHTML());
}

void handle_serv4on()
{
  state[4] = HIGH;
server.send(200, "text/html", SendHTML());
}

void handle_serv4off()
{
  state[4] = LOW;
server.send(200, "text/html", SendHTML());
}

void handle_serv4serv()
{
       serv[4] = 1;
       state[4] = !state[4];
       timing = millis();
       server.send(200, "text/html", SendHTML());
}

void handle_serv5on()
{
  state[5] = HIGH;
server.send(200, "text/html", SendHTML());
}

void handle_serv5off()
{
  state[5] = LOW;
server.send(200, "text/html", SendHTML());
}

void handle_serv5serv()
{
       serv[5] = 1;
       state[5] = !state[5];
       timing = millis();
       server.send(200, "text/html", SendHTML());
}
void handle_serv6on()
{
  state[6] = HIGH;
server.send(200, "text/html", SendHTML());
}

void handle_serv6off()
{
  state[6] = LOW;
server.send(200, "text/html", SendHTML());
}

void handle_serv6serv()
{
       serv[6] = 1;
       state[6] = !state[6];
       timing = millis();
       server.send(200, "text/html", SendHTML());
}

void handle_serv7on()
{
  state[7] = HIGH;
server.send(200, "text/html", SendHTML());
}

void handle_serv7off()
{
  state[7] = LOW;
server.send(200, "text/html", SendHTML());
}

void handle_serv7serv()
{
       serv[7] = 1;
       state[7] = !state[7];
       timing = millis();
       server.send(200, "text/html", SendHTML());
}

void handle_allon()
{
  for (int i = 0; i < 8; ++i) {
    state[i] = HIGH;
  }
  server.send(200, "text/html", SendHTML());
}

void handle_alloff()
{
  for (int i = 0; i < 8; ++i) {
    state[i] = LOW;
  }
  server.send(200, "text/html", SendHTML());
}

void handle_allserv()
{
  for (int i = 0; i < 8; ++i) {
    serv[i] = 1;
    state[i] = !state[i];
  }
  timing = millis();
  server.send(200, "text/html", SendHTML());
}

void handle_mainon()
{
  for (int i = 0; i < 3; ++i) {
    state[i] = HIGH;
  }
  server.send(200, "text/html", SendHTML());
}

void handle_mainoff()
{
  for (int i = 0; i < 3; ++i) {
    state[i] = LOW;
  }
  server.send(200, "text/html", SendHTML());
}

void handle_mainserv()
{
  for (int i = 0; i < 3; ++i) {
    serv[i] = 1;
    state[i] = !state[i];
  }
  timing = millis();
  server.send(200, "text/html", SendHTML());
}

void handle_polivon()
{
  for (int i = 3; i < 8; ++i) {
    state[i] = HIGH;
  }
  server.send(200, "text/html", SendHTML());
}

void handle_polivoff()
{
  for (int i = 3; i < 8; ++i) {
    state[i] = LOW;
  }
  server.send(200, "text/html", SendHTML());
}

void handle_polivserv()
{
  for (int i = 3; i < 8; ++i) {
    serv[i] = 1;
    state[i] = !state[i];
  }
  timing = millis();
  server.send(200, "text/html", SendHTML());
}

//String SendHTML(uint8_t serv0stat, uint8_t serv1stat, uint8_t serv2stat, uint8_t serv3stat, uint8_t serv4stat, uint8_t serv5stat, uint8_t serv6stat, uint8_t serv7stat)
String SendHTML()
{
  String ptr = "<!DOCTYPE html> <html>\n";
  ptr +="<head><meta name=\"viewport\" content=\"10;URL=/\" http-equiv=\"refresh\" charset=utf-8>\n";
  ptr +="<title>Water Valve Control</title>\n";
  ptr +="<style>html { font-family: Helvetica; display: inline-block; margin: 0px auto; text-align: center;}\n";
  ptr +="body{margin-top: 50px;} h1 {color: #444444;margin: 50px auto 30px;} h3 {color: #444444;margin-bottom: 50px;}\n";
  ptr +=".button {display: block;width: 80px;background-color: #1abc9c;border: none;color: white;padding: 13px 30px;text-decoration: none;font-size: 25px;margin: 0px auto 35px;cursor: pointer;border-radius: 4px;}\n";
  ptr +=".button-on {background-color: #1abc9c;}\n";
  ptr +=".button-on:active {background-color: #16a085;}\n";
  ptr +=".button-off {background-color: #34495e;}\n";
  ptr +=".button-off:active {background-color: #2c3e50;}\n";
  ptr +="p {font-size: 14px;color: #888;margin-bottom: 10px;}\n";
  ptr +="</style>\n";
  ptr +="</head>\n";
  ptr +="<body>\n";
  ptr +="<h1>Valve Web Server</h1>\n";
  ptr +="<h3>Using Access Point: ";
  ptr += WiFi.SSID();
  ptr += "<small>. Uptime: ";
  ptr += millis2time();
  ptr += "</small></h3>\n<table border=0 width=50% align=center>";

  ptr += "<tr><td align=left>До фильтра</td><td colspan=2><a class=\"button button-";
  
  if(state[0])
    ptr += "on\" href=\"/serv0off\">Open";
  else
    ptr += "off\" href=\"/serv0on\">Closed";

  ptr += "</a>\n<td><a class=\"button button-off\" href=\"/serv0serv\">Service</a></td></tr>\n";
  ptr += "<tr><td align=left>После фильтра</td><td colspan=2><a class=\"button button-";

  if(state[1])
    ptr += "on\" href=\"/serv1off\">Open";
  else
    ptr += "off\" href=\"/serv1on\">Closed";
  
  ptr += "</a>\n<td><a class=\"button button-off\" href=\"/serv1serv\">Service</a></td></tr>\n";
  ptr += "<tr><td align=left>Коллектор</td><td colspan=2><a class=\"button button-";

  if(state[2])
    ptr += "on\" href=\"/serv2off\">Open";
  else
    ptr += "off\" href=\"/serv2on\">Closed";

  ptr += "</a>\n<td><a class=\"button button-off\" href=\"/serv2serv\">Service</a></td></tr>\n";
  ptr += "<tr><td align=left>Лужайка центр</td><td colspan=2><a class=\"button button-";
  
  if(state[3])
    ptr += "on\" href=\"/serv3off\">Open";
  else
    ptr += "off\" href=\"/serv3on\">Closed";

  ptr += "</a>\n<td><a class=\"button button-off\" href=\"/serv3serv\">Service</a></td></tr>\n";
  ptr += "<tr><td align=left>Полив фронт дома</td><td colspan=2><a class=\"button button-";

  if(state[4])
    ptr += "on\" href=\"/serv4off\">Open";
  else
    ptr += "off\" href=\"/serv4on\">Closed";

    ptr += "</a>\n<td><a class=\"button button-off\" href=\"/serv4serv\">Service</a></td></tr>\n";
    ptr += "<tr><td align=left>Полив бассейн</td><td colspan=2><a class=\"button button-";

  if(state[5])
    ptr += "on\" href=\"/serv5off\">Open";
  else
    ptr += "off\" href=\"/serv5on\">Closed";

  ptr += "</a>\n<td><a class=\"button button-off\" href=\"/serv5serv\">Service</a></td></tr>\n";
  ptr += "<tr><td align=left>Полив самшиты</td><td colspan=2><a class=\"button button-";

  if(state[6])
    ptr += "on\" href=\"/serv6off\">Open";
  else
    ptr += "off\" href=\"/serv6on\">Closed";

  ptr += "</a>\n<td><a class=\"button button-off\" href=\"/serv6serv\">Service</a></td></tr>\n";
  ptr += "<tr><td align=left>Полив забор</td><td colspan=2><a class=\"button button-";

  if(state[7])
    ptr += "on\" href=\"/serv7off\">Open";
  else
    ptr += "off\" href=\"/serv7on\">Closed";

  ptr += "</a>\n<td><a class=\"button button-off\" href=\"/serv7serv\">Service</a></td></tr>\n";

  ptr += "<tr><td align=left width=50%>Все краны</td><td><a class=\"button button-off\" href=\"/allon\">Open</a></td>\n";
  ptr += "<td><a class=\"button button-off\" href=\"/alloff\">Close";
  ptr += "</a>\n<td><a class=\"button button-off\" href=\"/allserv\">Service</a></td></tr>\n";

  ptr += "<tr><td align=left width=50%>Главные краны</td><td><a class=\"button button-off\" href=\"/mainon\">Open</a></td>\n";
  ptr += "<td><a class=\"button button-off\" href=\"/mainoff\">Close";
  ptr += "</a>\n<td><a class=\"button button-off\" href=\"/mainserv\">Service</a></td></tr>\n";

  ptr += "<tr><td align=left width=50%>Полив</td><td><a class=\"button button-off\" href=\"/polivon\">Open</a></td>\n";
  ptr += "<td><a class=\"button button-off\" href=\"/polivoff\">Close";
  ptr += "</a>\n<td><a class=\"button button-off\" href=\"/polivserv\">Service</a></td></tr>\n";

  ptr += "</table>\n</body>\n</html>\n";
  return ptr;
}
